function run = getCostLetterA(run, param, k)
% implements a simple squared acceleration cost function with a penalty on
% the length of the parameter vector. Additionally, a via point has to be traversed at
% certain moment of time

    n_dof  = run(k).n_dof;
    n_traj = numel(run(k).dof(1).q);  % the length of a trajectory in time steps
    
%     for j=1:n_dof
%         costJoint(j).cost = [];
%     end
%  
    % Better to not call this independently (one call for each DoF) because depending on the cost
    % function, it may be better to have access to all joints simultaneously.
    costPerJoint  = inner_loop(run(k), param, n_dof, n_traj);
    
    % update 
    for j=1:n_dof
        run(k).dof(j).costHistory = costPerJoint(:,j);
        % costJoint(j).cost = [costJoint(j).cost     costPerJoint(:,j)];
    end        
    run(k).full_DoF_cost = sum(costPerJoint,2); 

end


function [costPerJoint] = inner_loop(run, param, n_dof, n_real)

    % compute cost
    Qacc       = param.repsCost.accel;
    Qvel       = param.repsCost.vel;
    QviaPoint  = param.repsCost.viaPoint;
    QparamSize = param.repsCost.paramSize;
    Qdeviation = param.repsCost.deviationFromMean;
    QstartGoal = param.repsCost.startGoal;    
     
    for j=1:n_dof
        
        Cost  = zeros(n_real,1);    
        
        % velocity cost
        velocity_cost = -run.dof(j).qdot(1:n_real);
        Cost  = Cost + Qvel*velocity_cost.^2;

        % acceleration cost
        acceleration_cost = -run.dof(j).qddot(1:n_real);
        Cost  = Cost + Qacc*acceleration_cost.^2;

        % punish size of changes in parameter
        parameter_size_cost = sum(  run.dof(j).theta_eps.^2  );
        parameter_size_cost = parameter_size_cost*ones(n_real,1);
        Cost  = Cost + QparamSize*parameter_size_cost;   
        
        % punish via point cost
        nviaPoints = numel( param.dof(j).viaPoint(:,1) );
        for kk =1:nviaPoints
            viaPoint = param.dof(j).viaPoint(kk,:);
            ind      = viaPoint(1);
            cost_t    = run.dof(j).q(ind)-viaPoint(2);
            Cost(ind) = Cost(ind) + QviaPoint(1)*cost_t^2;
        end
        
%         % punish start goal
%         nviaPoints = numel( param.dof(j).startGoal(:,1));
%         for kk =1:nviaPoints
%             startGoal = param.dof(j).startGoal(kk,:);
%             ind      = startGoal(1);
%             cost_t    = run.dof(j).q(ind)-startGoal(2);
%             Cost(ind) = Cost(ind) + QstartGoal(1)*cost_t^2;
%         end
%         
        
        % punish distance from mean
        if 1
            deviationFromMean = (param.repsCost.meanTraj(:,j)-run.dof(j).q);
            Cost  = Cost + Qdeviation*deviationFromMean.^2; 
        else
            % create anchor points that work like viapoints
            anchorIndexes = 1:10:numel(run.dof(j).q);
            for kk =1:numel(anchorIndexes)
                ia = anchorIndexes(kk);
                cost_t    = intialMean(ia,j)-run.dof(j).q(ia);
                Cost(ia) = Cost(ia) + Qdeviation*cost_t^2;
            end            
        end

        % punish collision
%         for j=1:n_dof
%             traj(:,j)     = run.dof(j).q;
%             traj_dot(:,j) = run.dof(j).qdot;
%         end
%         cost_collision = obstacleCost(param.obstacle, run.dof(j).q, run.dof(j).qdot );
%         cost_collision

        costPerJoint(:,j) = Cost;
        
        
    end % for each dof

    %punish collision
    for j=1:n_dof
        traj(:,j)     = run.dof(j).q;
        traj_dot(:,j) = run.dof(j).qdot;
    end
    %cost_collision = obstacleCost(param.obstacle, traj, traj_dot);
    cost_collision = obstacleCost(param.obstacle, traj);
    
    
    
    costPerJoint = bsxfun(@plus, costPerJoint,  param.repsCost.collision.*cost_collision);
        
end

% figurew(' ')
% plot(intialMean(:,j), 'r')
% plot(run.dof(j).q, 'g')





























