function collision_potential = obstacleCost(obs, traj, trajdot)
% dist = obstacleCost(obs, traj)
%
% For a given trajectory of a ball on the robot (given as "traj"), compute
% the history of distances in relation to all obstacles (nObstacles).
% 
% Automatically check and do not use the z dimension of the obstacle if 
% the trajectory is only 2D
% 
% INPUT
%   traj [nTraj, nDoF]
%   obj: the structure created from createObstacle
%
% OUTPUT
%   dist: [nTraj, nObstacles]
%

    nDoF = numel(traj(1,:));
    
    % if velocity is given, scale the distance by the norm of the velocity
    % vector. This punishes solutions that move at high speed when close to
    % the obstacle. See Stomp 2011 paper. The same velocity vector applies
    % to all obstacles.
    if exist('trajdot', 'var')
        velVector = trajdot(:, 1:nDoF);
        normVel   = distanceN( zeros(1,nDoF), velVector );
    else
        normVel = ones(numel(traj(:,1)),1);
    end

    safeDistance = (obs.minDist+obs.ballRadius);
    nObstacleVertices = numel(obs.verticesXYZ(:,1));
    for io =1:nObstacleVertices
        dist_tmp = dist_metric(  traj, obs.verticesXYZ(io,1:nDoF)  );
        collision_potential(:,io) = max(safeDistance-dist_tmp, 0).*normVel;
    end
    
end

function dist = dist_metric(sequenceOfBallPositions, queryObjectVertex)
    dist = distanceN(queryObjectVertex, sequenceOfBallPositions);
end













