function run = runRollOuts(sys, run, param, noise_mult, k)
% Run multiple roll-outs.
%
%       
    % make equations clean
    Gn     = sys.basis.Gn;
    Gndot  = sys.basis.Gndot;
    Gnddot = sys.basis.Gnddot;

        
    for j = 1:run(1).n_dof, % for each dmp independently

        % Select which type of covariance to use
        if param.reps.decreaseCovarianceAsPi2
            wCov = sys.dof(j).wCov(:,:,1); % use always the initial covariance.
        else
            wCov = sys.dof(j).wCov(:,:,end); % use the lates covariance updated by REPS
        end

        % perturb the weight vector
        epsilon = mvnrnd(zeros(1, sys.nBasis), noise_mult.*wCov);

        % check if should perturb the initial and final states
        if param.reps.shakeInitial_nWeights
            epsilon(1:param.reps.shakeInitial_nWeights) = mean(epsilon).*ones(1,param.reps.shakeInitial_nWeights);
        end
        if param.reps.shakeFinal_nWeights
            epsilon(end-param.reps.shakeFinal_nWeights+1:end) = mean(epsilon).*ones(1,param.reps.shakeFinal_nWeights);
        end            

        run(k).dof(j).q       = Gn*sys.dof(j).wMean(:,end)    + Gn*epsilon';
        run(k).dof(j).qdot    = Gndot*sys.dof(j).wMean(:,end) + Gndot*epsilon';
        run(k).dof(j).qddot   = Gnddot*sys.dof(j).wMean(:,end)+ Gnddot*epsilon';
        run(k).dof(j).epsilon = epsilon;
        run(k).dof(j).theta_eps   = sys.dof(j).wMean(:,end)' + epsilon;
    end
        

    
end
