function repsParam = userParamREPS()

    repsParam.KLBound     = 0.9;
    
    repsParam.n_rollOouts = 15; % use n_rollOouts to estimate the weights for the ML update.
    
    repsParam.n_updates   = 15; % update the parameters after n_rollOouts  were sampled.

    repsParam.n_reuse     = 5;
    
    repsParam.decreaseCovarianceAsPi2  = 1; % disable REPS updating of covariance and use the Pi2
                                            % approach of linearly decreasing the noise.
                                            
    repsParam.minNoiseMultiplierValue  = 0.01; % the minimum noise scaling factor that is achieved by 
                                               % the final iteration. Note that this value is only
                                               % effective when decreaseCovarianceAsPi2 is active.
                                          
    repsParam.useSTOMPCovariance     = 1; % Stomp-type of covariance that creates an ellipse of trajectories
                                          % around the initial guess. This is convenient for motion
                                          % planning problems where the initial and final positions
                                          % must not change. You should not use this for general RL
                                          % problems where the initial and final states must be
                                          % explored because there is basically no noise at the
                                          % extremities.
                                          
    
    % Initial value to scale the covariance matrix.  Heavily problem dependent.
    repsParam.covarInitialScale = 0.00005; % scales the A matrix for stomp-like covariance.
        
    % Anchor weights
    % This disables the updating of the first and last weights. It is suitable for STOMP like
    % problems where the initial and final states are given.
    % If you need to explore the initial and final states you must set them to zero and set the
    % parameters "shakeInitial_nWeights" and "shakeFinal_nWeights" accordingly.
    % This can be seen as a hack, but it makes the solution converge much faster and towards better
    % solutions.
    repsParam.anchorInitial_nWeights = 3;
    repsParam.anchorFinal_nWeights   = 3;
    
    
    % Shake initial weights
    % This is useful when you want to change the initial and goal positions. 
    repsParam.shakeInitial_nWeights = 0; % force noise on the first n weights.
    repsParam.shakeFinal_nWeights   = 0; % force noise on the last n weights.
    
    % check if some parameters are consistent
    if repsParam.anchorInitial_nWeights && repsParam.shakeInitial_nWeights
        error('Both anchorInitial_nWeights and shakeInitial_nWeights cannot be nonzero at the same time.');
    end
    if repsParam.anchorFinal_nWeights && repsParam.shakeFinal_nWeights
        error('Both anchorFinal_nWeights and shakeFinal_nWeights cannot be nonzero at the same time.');
    end    
    
end
