function sys = main_loop(param, sys, h)
%  sys = main_loop(param, sys, h)
%
%  INPUT:
%
%   param: contains several parameters for REPS, Obstacles, etc.
%   sys: this structure holds some useful information and the initial parameters for each DoF.
%        the fields sys.dof(j).wMean and sys.dof(j).wCov are updated incrementally so that it will
%        hold a sequence of updates. Therefore, in order to use the final solution you should use
%        sys.dof(j).wMean(:,end) and sys.dof(j).wCov(:,:,end)
%
%  OUPTUT
%
%   sys: the updated system structure.

    % create and extra parameter structure that is only used for evaluating the noiseless rollout.
    param_eval                     = param;  
    param_eval.reps.n_rollOouts    = 1;
    param_eval.reps.n_reuse = 0;      
    zero_noise = 0;
    
    % Creating initial promp structure
    run_eval = createMainStruc(sys(1).nDoF);

    % Create an equal number of structures that holds simulations
    run_noisy(1:param.reps.n_rollOouts) = run_eval;  % one data structure for each repetition

    % plot learning curve
    hfig.learningCurve = figurew('learning curve');
    plot([param.reps.n_updates param.reps.n_updates],[0  10],'r');
    xlabel('Number of roll outs');  ylabel('Cost');
    xlim([0 param.reps.n_updates]);
    set_fig_position([0.671 0.05 0.296 0.468]);

   
    for i= 1:param.reps.n_updates,

        tic
        % noiseless roll-out
        run_eval   = runRollOuts(sys, run_eval, param_eval, zero_noise, 1); 
        run_eval   = getCostLetterA(run_eval, param_eval, 1); % compute cost
        
        if param.reps.decreaseCovarianceAsPi2 % decrease noise linearly.
            noise_mult = getNoiseAsPi2(i,  param.reps.n_updates, param.reps.minNoiseMultiplierValue);
        else % keep noise scaling constant, but use REPS to adjust the covariance automatically
            noise_mult = 1 ;
        end
        
        
        % ===========================================
        % run roll-outs
        iroll = param.reps.n_reuse + 1; % 
        if i ==1,  iroll = 1;  end % first batch of roll-outs
        
        for p = iroll:param.reps.n_rollOouts
            run_noisy = runRollOuts(sys, run_noisy, param, noise_mult, p);               
            run_noisy = getCostLetterA(run_noisy, param, p);
            
            aux_cost_full_DoF(p) = sum(run_noisy(p).full_DoF_cost); % sum of cost trajectory of all DoFs
        end
        % ===========================================
        
        if 1 % plotting...
            try delete(hfig.del); end
            hfig.del = [];            
            %figure(hfig.traj); 
            figure(h.traj(1).fig);             
            for k=1:param.reps.n_rollOouts
                hfig.del = [hfig.del plot( run_noisy(k).dof(1).q,  sty('r', [], 0.5)  )];
            end            
            figure(h.traj(2).fig);             
            for k=1:param.reps.n_rollOouts
                hfig.del = [hfig.del plot( run_noisy(k).dof(2).q,  sty('r', [], 0.5)  )];
            end            
            figure(hfig.learningCurve); 
            plot(i, sum(run_eval.full_DoF_cost), sty('r', 's', [],[], 5) );
            
            figure(h.traj(3).fig);
            try delete(hAA); end; hAA = [];
            for k=1:param.reps.n_rollOouts
                hAA = [hAA plot(run_noisy(k).dof(1).q, run_noisy(k).dof(2).q,  sty('r', [], 1) )];
            end           
            drawnow; 
            pause(0.3);
        end
        
        % update parameters
        [sys]   = reps_wrapper(sys, run_noisy, param);   

        % re-use the best n_reuse solutions
        if (i > 1 && param.reps.n_reuse > 0) % importance sampling
            [v,inds]  = sort( aux_cost_full_DoF); % sort them by minimum to maximum cost
            for j=1:param.reps.n_reuse,
                run_temp = run_noisy(j);
                run_noisy(j)  = run_noisy(inds(j));
                run_noisy(inds(j)) = run_temp;
            end
        end                  
        fprintf('Update %g of %g. Update loop %g sec.\n', i+1, param.reps.n_updates, toc);

    end % updates
    
    

    
end



















